@@ -3,10 +3,10 @@
 # Author: Ewan Klein <ewan@inf.ed.ac.uk>
 #         Dan Garrette <dhgarrette@gmail.com>
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
-"""
+r"""
 Module for incrementally developing simple discourses, and checking for semantic ambiguity,
 consistency and informativeness.
 
@@ -42,28 +42,24 @@ The set of all threads for a discourse is the Cartesian product of all the readi
 (This is not intended to scale beyond very short discourses!) The method ``readings(filter=True)`` will only show
 those threads which are consistent (taking into account any background assumptions).
 """
-from __future__ import print_function
-from abc import ABCMeta, abstractmethod
-from six import add_metaclass
-import os
 
-from operator import and_, add
+import os
+from abc import ABCMeta, abstractmethod
 from functools import reduce
+from operator import add, and_
 
 from nltk.data import show_cfg
-from nltk.tag import RegexpTagger
-from nltk.parse import load_parser
-from nltk.parse.malt import MaltParser
-from nltk.sem.drt import resolve_anaphora, AnaphoraResolutionException
-from nltk.sem.glue import DrtGlue
-from nltk.sem.logic import Expression
-
 from nltk.inference.mace import MaceCommand
 from nltk.inference.prover9 import Prover9Command
+from nltk.parse import load_parser
+from nltk.parse.malt import MaltParser
+from nltk.sem.drt import AnaphoraResolutionException, resolve_anaphora
+from nltk.sem.glue import DrtGlue
+from nltk.sem.logic import Expression
+from nltk.tag import RegexpTagger
 
 
-@add_metaclass(ABCMeta)
-class ReadingCommand(object):
+class ReadingCommand(metaclass=ABCMeta):
     @abstractmethod
     def parse_to_readings(self, sentence):
         """
@@ -110,12 +106,15 @@ class CfgReadingCommand(ReadingCommand):
         :param gramfile: name of file where grammar can be loaded
         :type gramfile: str
         """
-        self._gramfile = (gramfile if gramfile else 'grammars/book_grammars/discourse.fcfg')
+        self._gramfile = (
+            gramfile if gramfile else "grammars/book_grammars/discourse.fcfg"
+        )
         self._parser = load_parser(self._gramfile)
 
     def parse_to_readings(self, sentence):
         """:see: ReadingCommand.parse_to_readings()"""
         from nltk.sem import root_semrep
+
         tokens = sentence.split()
         trees = self._parser.parse(tokens)
         return [root_semrep(tree) for tree in trees]
@@ -130,18 +129,21 @@ class CfgReadingCommand(ReadingCommand):
 
 
 class DrtGlueReadingCommand(ReadingCommand):
-    def __init__(self, semtype_file=None, remove_duplicates=False,
-                 depparser=None):
+    def __init__(self, semtype_file=None, remove_duplicates=False, depparser=None):
         """
         :param semtype_file: name of file where grammar can be loaded
         :param remove_duplicates: should duplicates be removed?
         :param depparser: the dependency parser
         """
         if semtype_file is None:
-            semtype_file = os.path.join('grammars', 'sample_grammars','drt_glue.semtype')
-        self._glue = DrtGlue(semtype_file=semtype_file,
-                             remove_duplicates=remove_duplicates,
-                             depparser=depparser)
+            semtype_file = os.path.join(
+                "grammars", "sample_grammars", "drt_glue.semtype"
+            )
+        self._glue = DrtGlue(
+            semtype_file=semtype_file,
+            remove_duplicates=remove_duplicates,
+            depparser=depparser,
+        )
 
     def parse_to_readings(self, sentence):
         """:see: ReadingCommand.parse_to_readings()"""
@@ -164,10 +166,11 @@ class DrtGlueReadingCommand(ReadingCommand):
         return expression.fol()
 
 
-class DiscourseTester(object):
+class DiscourseTester:
     """
     Check properties of an ongoing discourse.
     """
+
     def __init__(self, input, reading_command=None, background=None):
         """
         Initialize a ``DiscourseTester``.
@@ -178,14 +181,17 @@ class DiscourseTester(object):
         :type background: list(Expression)
         """
         self._input = input
-        self._sentences = dict([('s%s' % i, sent) for i, sent in enumerate(input)])
+        self._sentences = {"s%s" % i: sent for i, sent in enumerate(input)}
         self._models = None
         self._readings = {}
-        self._reading_command = (reading_command if reading_command else CfgReadingCommand())
+        self._reading_command = (
+            reading_command if reading_command else CfgReadingCommand()
+        )
         self._threads = {}
         self._filtered_threads = {}
         if background is not None:
             from nltk.sem.logic import Expression
+
             for e in background:
                 assert isinstance(e, Expression)
             self._background = background
@@ -201,9 +207,9 @@ class DiscourseTester(object):
         Display the list of sentences in the current discourse.
         """
         for id in sorted(self._sentences):
-            print("%s: %s" % (id, self._sentences[id]))
+            print(f"{id}: {self._sentences[id]}")
 
-    def add_sentence(self, sentence, informchk=False, consistchk=False,):
+    def add_sentence(self, sentence, informchk=False, consistchk=False):
         """
         Add a sentence to the current discourse.
 
@@ -223,11 +229,14 @@ class DiscourseTester(object):
                 for sent_reading in self._get_readings(sentence):
                     tp = Prover9Command(goal=sent_reading, assumptions=assumptions)
                     if tp.prove():
-                        print("Sentence '%s' under reading '%s':" % (sentence, str(sent_reading)))
+                        print(
+                            "Sentence '%s' under reading '%s':"
+                            % (sentence, str(sent_reading))
+                        )
                         print("Not informative relative to thread '%s'" % tid)
 
         self._input.append(sentence)
-        self._sentences = dict([('s%s' % i, sent) for i, sent in enumerate(self._input)])
+        self._sentences = {"s%s" % i: sent for i, sent in enumerate(self._input)}
         # check whether adding the new sentence to the discourse preserves consistency (i.e. a model can be found for the combined set of
         # of assumptions
         if consistchk:
@@ -246,10 +255,13 @@ class DiscourseTester(object):
         try:
             self._input.remove(sentence)
         except ValueError:
-            print("Retraction failed. The sentence '%s' is not part of the current discourse:" % sentence)
+            print(
+                "Retraction failed. The sentence '%s' is not part of the current discourse:"
+                % sentence
+            )
             self.sentences()
             return None
-        self._sentences = dict([('s%s' % i, sent) for i, sent in enumerate(self._input)])
+        self._sentences = {"s%s" % i: sent for i, sent in enumerate(self._input)}
         self.readings(verbose=False)
         if verbose:
             print("Current sentences are ")
@@ -282,8 +294,10 @@ class DiscourseTester(object):
         for sid in sorted(self._sentences):
             sentence = self._sentences[sid]
             readings = self._get_readings(sentence)
-            self._readings[sid] = dict([("%s-r%s" % (sid, rid), reading.simplify())
-                                                        for rid, reading in enumerate(sorted(readings, key=str))])
+            self._readings[sid] = {
+                f"{sid}-r{rid}": reading.simplify()
+                for rid, reading in enumerate(sorted(readings, key=str))
+            }
 
     def _construct_threads(self):
         """
@@ -293,12 +307,12 @@ class DiscourseTester(object):
         thread_list = [[]]
         for sid in sorted(self._readings):
             thread_list = self.multiply(thread_list, sorted(self._readings[sid]))
-        self._threads = dict([("d%s" % tid, thread) for tid, thread in enumerate(thread_list)])
+        self._threads = {"d%s" % tid: thread for tid, thread in enumerate(thread_list)}
         # re-initialize the filtered threads
         self._filtered_threads = {}
         # keep the same ids, but only include threads which get models
         consistency_checked = self._check_consistency(self._threads)
-        for (tid, thread) in self._threads.items():
+        for tid, thread in self._threads.items():
             if (tid, True) in consistency_checked:
                 self._filtered_threads[tid] = thread
 
@@ -313,34 +327,42 @@ class DiscourseTester(object):
         else:
             for sid in sorted(self._readings):
                 print()
-                print('%s readings:' % sid)
-                print() #'-' * 30
+                print("%s readings:" % sid)
+                print()  #'-' * 30
                 for rid in sorted(self._readings[sid]):
                     lf = self._readings[sid][rid]
-                    print("%s: %s" % (rid, lf.normalize()))
+                    print(f"{rid}: {lf.normalize()}")
 
     def _show_threads(self, filter=False, show_thread_readings=False):
         """
         Print out the value of ``self._threads`` or ``self._filtered_hreads``
         """
-        threads = (self._filtered_threads if filter else self._threads)
+        threads = self._filtered_threads if filter else self._threads
         for tid in sorted(threads):
             if show_thread_readings:
-                readings = [self._readings[rid.split('-')[0]][rid]
-                            for rid in self._threads[tid]]
+                readings = [
+                    self._readings[rid.split("-")[0]][rid] for rid in self._threads[tid]
+                ]
                 try:
-                    thread_reading = ": %s" % \
-                              self._reading_command.combine_readings(readings).normalize()
+                    thread_reading = (
+                        ": %s"
+                        % self._reading_command.combine_readings(readings).normalize()
+                    )
                 except Exception as e:
-                    thread_reading = ': INVALID: %s' % e.__class__.__name__
+                    thread_reading = ": INVALID: %s" % e.__class__.__name__
             else:
-                thread_reading = ''
+                thread_reading = ""
 
             print("%s:" % tid, self._threads[tid], thread_reading)
 
-
-    def readings(self, sentence=None, threaded=False, verbose=True,
-                 filter=False, show_thread_readings=False):
+    def readings(
+        self,
+        sentence=None,
+        threaded=False,
+        verbose=True,
+        filter=False,
+        show_thread_readings=False,
+    ):
         """
         Construct and show the readings of the discourse (or of a single sentence).
 
@@ -360,8 +382,9 @@ class DiscourseTester(object):
             if not threaded:
                 self._show_readings(sentence=sentence)
             else:
-                self._show_threads(filter=filter,
-                                   show_thread_readings=show_thread_readings)
+                self._show_threads(
+                    filter=filter, show_thread_readings=show_thread_readings
+                )
 
     def expand_threads(self, thread_id, threads=None):
         """
@@ -376,8 +399,11 @@ class DiscourseTester(object):
         """
         if threads is None:
             threads = self._threads
-        return [(rid, self._readings[sid][rid]) for rid in threads[thread_id] for sid in rid.split('-')[:1]]
-
+        return [
+            (rid, self._readings[sid][rid])
+            for rid in threads[thread_id]
+            for sid in rid.split("-")[:1]
+        ]
 
     ###############################
     # Models and Background
@@ -386,8 +412,15 @@ class DiscourseTester(object):
     def _check_consistency(self, threads, show=False, verbose=False):
         results = []
         for tid in sorted(threads):
-            assumptions = [reading for (rid, reading) in self.expand_threads(tid, threads=threads)]
-            assumptions = list(map(self._reading_command.to_fol, self._reading_command.process_thread(assumptions)))
+            assumptions = [
+                reading for (rid, reading) in self.expand_threads(tid, threads=threads)
+            ]
+            assumptions = list(
+                map(
+                    self._reading_command.to_fol,
+                    self._reading_command.process_thread(assumptions),
+                )
+            )
             if assumptions:
                 assumptions += self._background
                 # if Mace4 finds a model, it always seems to find it quickly
@@ -405,7 +438,7 @@ class DiscourseTester(object):
                         print(a)
                     spacer(80)
                 if modelfound:
-                    print(mb.model(format='cooked'))
+                    print(mb.model(format="cooked"))
                 else:
                     print("No model found!\n")
         return results
@@ -420,20 +453,22 @@ class DiscourseTester(object):
         """
         self._construct_readings()
         self._construct_threads()
-        threads = ({thread_id: self._threads[thread_id]} if thread_id else self._threads)
+        threads = {thread_id: self._threads[thread_id]} if thread_id else self._threads
 
-        for (tid, modelfound) in self._check_consistency(threads, show=show, verbose=verbose):
+        for tid, modelfound in self._check_consistency(
+            threads, show=show, verbose=verbose
+        ):
             idlist = [rid for rid in threads[tid]]
 
             if not modelfound:
-                print("Inconsistent discourse: %s %s:" % (tid, idlist))
+                print(f"Inconsistent discourse: {tid} {idlist}:")
                 for rid, reading in self.expand_threads(tid):
-                    print("    %s: %s" % (rid, reading.normalize()))
+                    print(f"    {rid}: {reading.normalize()}")
                 print()
             else:
-                print("Consistent discourse: %s %s:" % (tid, idlist))
+                print(f"Consistent discourse: {tid} {idlist}:")
                 for rid, reading in self.expand_threads(tid):
-                    print("    %s: %s" % (rid, reading.normalize()))
+                    print(f"    {rid}: {reading.normalize()}")
                 print()
 
     def add_background(self, background, verbose=False):
@@ -445,13 +480,14 @@ class DiscourseTester(object):
         :type background: list(Expression)
         """
         from nltk.sem.logic import Expression
-        for (count, e) in enumerate(background):
+
+        for count, e in enumerate(background):
             assert isinstance(e, Expression)
             if verbose:
                 print("Adding assumption %s to background" % count)
             self._background.append(e)
 
-        #update the state
+        # update the state
         self._construct_readings()
         self._construct_threads()
 
@@ -462,7 +498,7 @@ class DiscourseTester(object):
         for e in self._background:
             print(str(e))
 
-   ###############################
+    ###############################
     # Misc
     ###############################
 
@@ -489,11 +525,6 @@ class DiscourseTester(object):
                 result.append(new)
         return result
 
-#multiply = DiscourseTester.multiply
-#L1 = [['A'], ['B']]
-#L2 = ['a', 'b', 'c']
-#print multiply(L1,L2)
-
 
 def load_fol(s):
     """
@@ -508,12 +539,12 @@ def load_fol(s):
     statements = []
     for linenum, line in enumerate(s.splitlines()):
         line = line.strip()
-        if line.startswith('#') or line == '':
+        if line.startswith("#") or line == "":
             continue
         try:
             statements.append(Expression.fromstring(line))
-        except Exception:
-            raise ValueError('Unable to parse line %s: %s' % (linenum, line))
+        except Exception as e:
+            raise ValueError(f"Unable to parse line {linenum}: {line}") from e
     return statements
 
 
@@ -524,8 +555,9 @@ def discourse_demo(reading_command=None):
     """
     Illustrate the various methods of ``DiscourseTester``
     """
-    dt = DiscourseTester(['A boxer walks', 'Every boxer chases a girl'],
-                         reading_command)
+    dt = DiscourseTester(
+        ["A boxer walks", "Every boxer chases a girl"], reading_command
+    )
     dt.models()
     print()
     # dt.grammar()
@@ -536,33 +568,36 @@ def discourse_demo(reading_command=None):
     print()
     dt.readings(threaded=True)
     print()
-    dt.models('d1')
-    dt.add_sentence('John is a boxer')
+    dt.models("d1")
+    dt.add_sentence("John is a boxer")
     print()
     dt.sentences()
     print()
     dt.readings(threaded=True)
     print()
-    dt = DiscourseTester(['A student dances', 'Every student is a person'],
-                         reading_command)
+    dt = DiscourseTester(
+        ["A student dances", "Every student is a person"], reading_command
+    )
     print()
-    dt.add_sentence('No person dances', consistchk=True)
+    dt.add_sentence("No person dances", consistchk=True)
     print()
     dt.readings()
     print()
-    dt.retract_sentence('No person dances', verbose=True)
+    dt.retract_sentence("No person dances", verbose=True)
     print()
     dt.models()
     print()
-    dt.readings('A person dances')
+    dt.readings("A person dances")
     print()
-    dt.add_sentence('A person dances', informchk=True)
-    dt = DiscourseTester(['Vincent is a boxer', 'Fido is a boxer',
-                          'Vincent is married', 'Fido barks'],
-                         reading_command)
+    dt.add_sentence("A person dances", informchk=True)
+    dt = DiscourseTester(
+        ["Vincent is a boxer", "Fido is a boxer", "Vincent is married", "Fido barks"],
+        reading_command,
+    )
     dt.readings(filter=True)
     import nltk.data
-    background_file = os.path.join('grammars', 'book_grammars', 'background.fol')
+
+    background_file = os.path.join("grammars", "book_grammars", "background.fol")
     background = nltk.data.load(background_file)
 
     print()
@@ -578,8 +613,7 @@ def drt_discourse_demo(reading_command=None):
     """
     Illustrate the various methods of ``DiscourseTester``
     """
-    dt = DiscourseTester(['every dog chases a boy', 'he runs'],
-                         reading_command)
+    dt = DiscourseTester(["every dog chases a boy", "he runs"], reading_command)
     dt.models()
     print()
     dt.sentences()
@@ -592,21 +626,26 @@ def drt_discourse_demo(reading_command=None):
 
 
 def spacer(num=30):
-    print('-' * num)
+    print("-" * num)
 
 
 def demo():
     discourse_demo()
 
-    tagger = RegexpTagger([('^(chases|runs)$', 'VB'),
-                           ('^(a)$', 'ex_quant'),
-                           ('^(every)$', 'univ_quant'),
-                           ('^(dog|boy)$', 'NN'),
-                           ('^(he)$', 'PRP')])
+    tagger = RegexpTagger(
+        [
+            ("^(chases|runs)$", "VB"),
+            ("^(a)$", "ex_quant"),
+            ("^(every)$", "univ_quant"),
+            ("^(dog|boy)$", "NN"),
+            ("^(he)$", "PRP"),
+        ]
+    )
     depparser = MaltParser(tagger=tagger)
-    drt_discourse_demo(DrtGlueReadingCommand(remove_duplicates=False,
-                                             depparser=depparser))
+    drt_discourse_demo(
+        DrtGlueReadingCommand(remove_duplicates=False, depparser=depparser)
+    )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     demo()
