@@ -1,44 +1,46 @@
 # Natural Language Toolkit: Dependency Grammars
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2024 NLTK Project
 # Author: Jason Narad <jason.narad@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com> (modifications)
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 #
 
 """
 Tools for reading and writing dependency trees.
 The input is assumed to be in Malt-TAB format
-(http://stp.lingfil.uu.se/~nivre/research/MaltXML.html).
+(https://stp.lingfil.uu.se/~nivre/research/MaltXML.html).
 """
-from __future__ import print_function, unicode_literals
 
+import subprocess
+import warnings
 from collections import defaultdict
 from itertools import chain
 from pprint import pformat
-import subprocess
-import warnings
-
-from six import string_types
 
+from nltk.internals import find_binary
 from nltk.tree import Tree
-from nltk.compat import python_2_unicode_compatible
-
 
 #################################################################
 # DependencyGraph Class
 #################################################################
 
 
-@python_2_unicode_compatible
-class DependencyGraph(object):
+class DependencyGraph:
     """
     A container for the nodes and labelled edges of a dependency structure.
     """
 
-    def __init__(self, tree_str=None, cell_extractor=None, zero_based=False, cell_separator=None, top_relation_label='ROOT'):
+    def __init__(
+        self,
+        tree_str=None,
+        cell_extractor=None,
+        zero_based=False,
+        cell_separator=None,
+        top_relation_label="ROOT",
+    ):
         """Dependency graph.
 
         We place a dummy `TOP` node with the index 0, since the root node is
@@ -50,31 +52,27 @@ class DependencyGraph(object):
         zpar).
 
         :param str cell_separator: the cell separator. If not provided, cells
-        are split by whitespace.
+            are split by whitespace.
 
         :param str top_relation_label: the label by which the top relation is
-        identified, for examlple, `ROOT`, `null` or `TOP`.
-
+            identified, for examlple, `ROOT`, `null` or `TOP`.
         """
-        self.nodes = defaultdict(lambda:  {'address': None,
-                                           'word': None,
-                                           'lemma': None,
-                                           'ctag': None,
-                                           'tag': None,
-                                           'feats': None,
-                                           'head': None,
-                                           'deps': defaultdict(list),
-                                           'rel': None,
-                                           })
-
-        self.nodes[0].update(
-            {
-                'ctag': 'TOP',
-                'tag': 'TOP',
-                'address': 0,
+        self.nodes = defaultdict(
+            lambda: {
+                "address": None,
+                "word": None,
+                "lemma": None,
+                "ctag": None,
+                "tag": None,
+                "feats": None,
+                "head": None,
+                "deps": defaultdict(list),
+                "rel": None,
             }
         )
 
+        self.nodes[0].update({"ctag": "TOP", "tag": "TOP", "address": 0})
+
         self.root = None
 
         if tree_str:
@@ -100,23 +98,22 @@ class DependencyGraph(object):
         """
         for node in self.nodes.values():
             new_deps = []
-            for dep in node['deps']:
+            for dep in node["deps"]:
                 if dep in originals:
                     new_deps.append(redirect)
                 else:
                     new_deps.append(dep)
-            node['deps'] = new_deps
+            node["deps"] = new_deps
 
     def add_arc(self, head_address, mod_address):
         """
         Adds an arc from the node specified by head_address to the
         node specified by the mod address.
         """
-        relation = self.nodes[mod_address]['rel']
-        self.nodes[head_address]['deps'].setdefault(relation, [])
-        self.nodes[head_address]['deps'][relation].append(mod_address)
-        #self.nodes[head_address]['deps'].append(mod_address)
-
+        relation = self.nodes[mod_address]["rel"]
+        self.nodes[head_address]["deps"].setdefault(relation, [])
+        self.nodes[head_address]["deps"][relation].append(mod_address)
+        # self.nodes[head_address]['deps'].append(mod_address)
 
     def connect_graph(self):
         """
@@ -125,11 +122,11 @@ class DependencyGraph(object):
         """
         for node1 in self.nodes.values():
             for node2 in self.nodes.values():
-                if node1['address'] != node2['address'] and node2['rel'] != 'TOP':
-                    relation = node2['rel']
-                    node1['deps'].setdefault(relation, [])
-                    node1['deps'][relation].append(node2['address'])
-                    #node1['deps'].append(node2['address'])
+                if node1["address"] != node2["address"] and node2["rel"] != "TOP":
+                    relation = node2["rel"]
+                    node1["deps"].setdefault(relation, [])
+                    node1["deps"][relation].append(node2["address"])
+                    # node1['deps'].append(node2['address'])
 
     def get_by_address(self, node_address):
         """Return the node with the given address."""
@@ -166,26 +163,31 @@ class DependencyGraph(object):
 
         """
         # Start the digraph specification
-        s = 'digraph G{\n'
-        s += 'edge [dir=forward]\n'
-        s += 'node [shape=plaintext]\n'
+        s = "digraph G{\n"
+        s += "edge [dir=forward]\n"
+        s += "node [shape=plaintext]\n"
 
         # Draw the remaining nodes
-        for node in sorted(self.nodes.values(), key=lambda v: v['address']):
-            s += '\n%s [label="%s (%s)"]' % (node['address'], node['address'], node['word'])
-            for rel, deps in node['deps'].items():
+        for node in sorted(self.nodes.values(), key=lambda v: v["address"]):
+            s += '\n{} [label="{} ({})"]'.format(
+                node["address"],
+                node["address"],
+                node["word"],
+            )
+            for rel, deps in node["deps"].items():
                 for dep in deps:
                     if rel is not None:
-                        s += '\n%s -> %s [label="%s"]' % (node['address'], dep, rel)
+                        s += '\n{} -> {} [label="{}"]'.format(node["address"], dep, rel)
                     else:
-                        s += '\n%s -> %s ' % (node['address'], dep)
+                        s += "\n{} -> {} ".format(node["address"], dep)
         s += "\n}"
 
         return s
 
     def _repr_svg_(self):
         """Show SVG representation of the transducer (IPython magic).
-
+        >>> from nltk.test.setup_fixt import check_binary
+        >>> check_binary('dot')
         >>> dg = DependencyGraph(
         ...     'John N 2\\n'
         ...     'loves V 0\\n'
@@ -196,40 +198,26 @@ class DependencyGraph(object):
 
         """
         dot_string = self.to_dot()
-
-        try:
-            process = subprocess.Popen(
-                ['dot', '-Tsvg'],
-                stdin=subprocess.PIPE,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-                universal_newlines=True,
-            )
-        except OSError:
-            raise Exception('Cannot find the dot binary from Graphviz package')
-        out, err = process.communicate(dot_string)
-        if err:
-            raise Exception(
-                'Cannot create svg representation by running dot from string: {}'
-                ''.format(dot_string))
-        return out
+        return dot2img(dot_string)
 
     def __str__(self):
         return pformat(self.nodes)
 
     def __repr__(self):
-        return "<DependencyGraph with {0} nodes>".format(len(self.nodes))
+        return f"<DependencyGraph with {len(self.nodes)} nodes>"
 
     @staticmethod
-    def load(filename, zero_based=False, cell_separator=None, top_relation_label='ROOT'):
+    def load(
+        filename, zero_based=False, cell_separator=None, top_relation_label="ROOT"
+    ):
         """
         :param filename: a name of a file in Malt-TAB format
         :param zero_based: nodes in the input file are numbered starting from 0
-        rather than 1 (as produced by, e.g., zpar)
+            rather than 1 (as produced by, e.g., zpar)
         :param str cell_separator: the cell separator. If not provided, cells
-        are split by whitespace.
+            are split by whitespace.
         :param str top_relation_label: the label by which the top relation is
-        identified, for examlple, `ROOT`, `null` or `TOP`.
+            identified, for examlple, `ROOT`, `null` or `TOP`.
 
         :return: a list of DependencyGraphs
 
@@ -242,7 +230,7 @@ class DependencyGraph(object):
                     cell_separator=cell_separator,
                     top_relation_label=top_relation_label,
                 )
-                for tree_str in infile.read().split('\n\n')
+                for tree_str in infile.read().split("\n\n")
             ]
 
     def left_children(self, node_index):
@@ -250,8 +238,8 @@ class DependencyGraph(object):
         Returns the number of left children under the node specified
         by the given address.
         """
-        children = chain.from_iterable(self.nodes[node_index]['deps'].values())
-        index = self.nodes[node_index]['address']
+        children = chain.from_iterable(self.nodes[node_index]["deps"].values())
+        index = self.nodes[node_index]["address"]
         return sum(1 for c in children if c < index)
 
     def right_children(self, node_index):
@@ -259,15 +247,22 @@ class DependencyGraph(object):
         Returns the number of right children under the node specified
         by the given address.
         """
-        children = chain.from_iterable(self.nodes[node_index]['deps'].values())
-        index = self.nodes[node_index]['address']
+        children = chain.from_iterable(self.nodes[node_index]["deps"].values())
+        index = self.nodes[node_index]["address"]
         return sum(1 for c in children if c > index)
 
     def add_node(self, node):
-        if not self.contains_address(node['address']):
-            self.nodes[node['address']].update(node)
+        if not self.contains_address(node["address"]):
+            self.nodes[node["address"]].update(node)
 
-    def _parse(self, input_, cell_extractor=None, zero_based=False, cell_separator=None, top_relation_label='ROOT'):
+    def _parse(
+        self,
+        input_,
+        cell_extractor=None,
+        zero_based=False,
+        cell_separator=None,
+        top_relation_label="ROOT",
+    ):
         """Parse a sentence.
 
         :param extractor: a function that given a tuple of cells returns a
@@ -284,11 +279,11 @@ class DependencyGraph(object):
 
         def extract_3_cells(cells, index):
             word, tag, head = cells
-            return index, word, word, tag, tag, '', head, ''
+            return index, word, word, tag, tag, "", head, ""
 
         def extract_4_cells(cells, index):
             word, tag, head, rel = cells
-            return index, word, word, tag, tag, '', head, rel
+            return index, word, word, tag, tag, "", head, rel
 
         def extract_7_cells(cells, index):
             line_index, word, lemma, tag, _, head, rel = cells
@@ -297,7 +292,7 @@ class DependencyGraph(object):
             except ValueError:
                 # index can't be parsed as an integer, use default
                 pass
-            return index, word, lemma, tag, tag, '', head, rel
+            return index, word, lemma, tag, tag, "", head, rel
 
         def extract_10_cells(cells, index):
             line_index, word, lemma, ctag, tag, feats, head, rel, _, _ = cells
@@ -315,8 +310,8 @@ class DependencyGraph(object):
             10: extract_10_cells,
         }
 
-        if isinstance(input_, string_types):
-            input_ = (line for line in input_.split('\n'))
+        if isinstance(input_, str):
+            input_ = (line for line in input_.split("\n"))
 
         lines = (l.rstrip() for l in input_)
         lines = (l for l in lines if l)
@@ -332,21 +327,23 @@ class DependencyGraph(object):
             if cell_extractor is None:
                 try:
                     cell_extractor = extractors[cell_number]
-                except KeyError:
+                except KeyError as e:
                     raise ValueError(
-                        'Number of tab-delimited fields ({0}) not supported by '
-                        'CoNLL(10) or Malt-Tab(4) format'.format(cell_number)
-                    )
+                        "Number of tab-delimited fields ({}) not supported by "
+                        "CoNLL(10) or Malt-Tab(4) format".format(cell_number)
+                    ) from e
 
             try:
-                index, word, lemma, ctag, tag, feats, head, rel = cell_extractor(cells, index)
+                index, word, lemma, ctag, tag, feats, head, rel = cell_extractor(
+                    cells, index
+                )
             except (TypeError, ValueError):
                 # cell_extractor doesn't take 2 arguments or doesn't return 8
                 # values; assume the cell_extractor is an older external
                 # extractor and doesn't accept or return an index.
                 word, lemma, ctag, tag, feats, head, rel = cell_extractor(cells)
 
-            if head == '_':
+            if head == "_":
                 continue
 
             head = int(head)
@@ -355,48 +352,47 @@ class DependencyGraph(object):
 
             self.nodes[index].update(
                 {
-                    'address': index,
-                    'word': word,
-                    'lemma': lemma,
-                    'ctag': ctag,
-                    'tag': tag,
-                    'feats': feats,
-                    'head': head,
-                    'rel': rel,
+                    "address": index,
+                    "word": word,
+                    "lemma": lemma,
+                    "ctag": ctag,
+                    "tag": tag,
+                    "feats": feats,
+                    "head": head,
+                    "rel": rel,
                 }
             )
 
             # Make sure that the fake root node has labeled dependencies.
             if (cell_number == 3) and (head == 0):
                 rel = top_relation_label
-            self.nodes[head]['deps'][rel].append(index)
+            self.nodes[head]["deps"][rel].append(index)
 
-        if self.nodes[0]['deps'][top_relation_label]:
-            root_address = self.nodes[0]['deps'][top_relation_label][0]
+        if self.nodes[0]["deps"][top_relation_label]:
+            root_address = self.nodes[0]["deps"][top_relation_label][0]
             self.root = self.nodes[root_address]
             self.top_relation_label = top_relation_label
         else:
             warnings.warn(
-                "The graph doesn't contain a node "
-                "that depends on the root element."
+                "The graph doesn't contain a node " "that depends on the root element."
             )
 
     def _word(self, node, filter=True):
-        w = node['word']
+        w = node["word"]
         if filter:
-            if w != ',':
+            if w != ",":
                 return w
         return w
 
     def _tree(self, i):
-        """ Turn dependency graphs into NLTK trees.
+        """Turn dependency graphs into NLTK trees.
 
         :param int i: index of a node
         :return: either a word (if the indexed node is a leaf) or a ``Tree``.
         """
         node = self.get_by_address(i)
-        word = node['word']
-        deps = sorted(chain.from_iterable(node['deps'].values()))
+        word = node["word"]
+        deps = sorted(chain.from_iterable(node["deps"].values()))
 
         if deps:
             return Tree(word, [self._tree(dep) for dep in deps])
@@ -410,8 +406,8 @@ class DependencyGraph(object):
         """
         node = self.root
 
-        word = node['word']
-        deps = sorted(chain.from_iterable(node['deps'].values()))
+        word = node["word"]
+        deps = sorted(chain.from_iterable(node["deps"].values()))
         return Tree(word, [self._tree(dep) for dep in deps])
 
     def triples(self, node=None):
@@ -423,22 +419,21 @@ class DependencyGraph(object):
         if not node:
             node = self.root
 
-        head = (node['word'], node['ctag'])
-        for i in sorted(chain.from_iterable(node['deps'].values())):
+        head = (node["word"], node["ctag"])
+        for i in sorted(chain.from_iterable(node["deps"].values())):
             dep = self.get_by_address(i)
-            yield (head, dep['rel'], (dep['word'], dep['ctag']))
-            for triple in self.triples(node=dep):
-                yield triple
+            yield (head, dep["rel"], (dep["word"], dep["ctag"]))
+            yield from self.triples(node=dep)
 
     def _hd(self, i):
         try:
-            return self.nodes[i]['head']
+            return self.nodes[i]["head"]
         except IndexError:
             return None
 
     def _rel(self, i):
         try:
-            return self.nodes[i]['rel']
+            return self.nodes[i]["rel"]
         except IndexError:
             return None
 
@@ -466,14 +461,14 @@ class DependencyGraph(object):
         >>> cyclic_dg.root = top
 
         >>> cyclic_dg.contains_cycle()
-        [3, 1, 2, 4]
+        [1, 2, 4, 3]
 
         """
         distances = {}
 
         for node in self.nodes.values():
-            for dep in node['deps']:
-                key = tuple([node['address'], dep])
+            for dep in node["deps"]:
+                key = tuple([node["address"], dep])
                 distances[key] = 1
 
         for _ in self.nodes:
@@ -494,13 +489,13 @@ class DependencyGraph(object):
         return False  # return []?
 
     def get_cycle_path(self, curr_node, goal_node_index):
-        for dep in curr_node['deps']:
+        for dep in curr_node["deps"]:
             if dep == goal_node_index:
-                return [curr_node['address']]
-        for dep in curr_node['deps']:
+                return [curr_node["address"]]
+        for dep in curr_node["deps"]:
             path = self.get_cycle_path(self.get_by_address(dep), goal_node_index)
             if len(path) > 0:
-                path.insert(0, curr_node['address'])
+                path.insert(0, curr_node["address"])
                 return path
         return []
 
@@ -514,18 +509,24 @@ class DependencyGraph(object):
         """
 
         if style == 3:
-            template = '{word}\t{tag}\t{head}\n'
+            template = "{word}\t{tag}\t{head}\n"
         elif style == 4:
-            template = '{word}\t{tag}\t{head}\t{rel}\n'
+            template = "{word}\t{tag}\t{head}\t{rel}\n"
         elif style == 10:
-            template = '{i}\t{word}\t{lemma}\t{ctag}\t{tag}\t{feats}\t{head}\t{rel}\t_\t_\n'
+            template = (
+                "{i}\t{word}\t{lemma}\t{ctag}\t{tag}\t{feats}\t{head}\t{rel}\t_\t_\n"
+            )
         else:
             raise ValueError(
-                'Number of tab-delimited fields ({0}) not supported by '
-                'CoNLL(10) or Malt-Tab(4) format'.format(style)
+                "Number of tab-delimited fields ({}) not supported by "
+                "CoNLL(10) or Malt-Tab(4) format".format(style)
             )
 
-        return ''.join(template.format(i=i, **node) for i, node in sorted(self.nodes.items()) if node['tag'] != 'TOP')
+        return "".join(
+            template.format(i=i, **node)
+            for i, node in sorted(self.nodes.items())
+            if node["tag"] != "TOP"
+        )
 
     def nx_graph(self):
         """Convert the data in a ``nodelist`` into a networkx labeled directed graph."""
@@ -533,12 +534,11 @@ class DependencyGraph(object):
 
         nx_nodelist = list(range(1, len(self.nodes)))
         nx_edgelist = [
-            (n, self._hd(n), self._rel(n))
-            for n in nx_nodelist if self._hd(n)
+            (n, self._hd(n), self._rel(n)) for n in nx_nodelist if self._hd(n)
         ]
         self.nx_labels = {}
         for n in nx_nodelist:
-            self.nx_labels[n] = self.nodes[n]['word']
+            self.nx_labels[n] = self.nodes[n]["word"]
 
         g = networkx.MultiDiGraph()
         g.add_nodes_from(nx_nodelist)
@@ -547,6 +547,43 @@ class DependencyGraph(object):
         return g
 
 
+def dot2img(dot_string, t="svg"):
+    """
+    Create image representation fom dot_string, using the 'dot' program
+    from the Graphviz package.
+
+    Use the 't' argument to specify the image file format, for ex. 'jpeg', 'eps',
+    'json', 'png' or 'webp' (Running 'dot -T:' lists all available formats).
+
+    Note that the "capture_output" option of subprocess.run() is only available
+    with text formats (like svg), but not with binary image formats (like png).
+    """
+
+    try:
+        find_binary("dot")
+        try:
+            if t in ["dot", "dot_json", "json", "svg"]:
+                proc = subprocess.run(
+                    ["dot", "-T%s" % t],
+                    capture_output=True,
+                    input=dot_string,
+                    text=True,
+                )
+            else:
+                proc = subprocess.run(
+                    ["dot", "-T%s" % t],
+                    input=bytes(dot_string, encoding="utf8"),
+                )
+            return proc.stdout
+        except:
+            raise Exception(
+                "Cannot create image representation by running dot from string: {}"
+                "".format(dot_string)
+            )
+    except OSError as e:
+        raise Exception("Cannot find the dot binary from Graphviz package") from e
+
+
 class DependencyGraphError(Exception):
     """Dependency graph exception."""
 
@@ -563,7 +600,8 @@ def malt_demo(nx=False):
     A demonstration of the result of reading a dependency
     version of the first sentence of the Penn Treebank.
     """
-    dg = DependencyGraph("""Pierre  NNP     2       NMOD
+    dg = DependencyGraph(
+        """Pierre  NNP     2       NMOD
 Vinken  NNP     8       SUB
 ,       ,       2       P
 61      CD      5       NMOD
@@ -581,7 +619,8 @@ director        NN      12      PMOD
 Nov.    NNP     9       VMOD
 29      CD      16      NMOD
 .       .       9       VMOD
-""")
+"""
+    )
     tree = dg.tree()
     tree.pprint()
     if nx:
@@ -597,7 +636,7 @@ Nov.    NNP     9       VMOD
         networkx.draw_networkx_labels(g, pos, dg.nx_labels)
         pylab.xticks([])
         pylab.yticks([])
-        pylab.savefig('tree.png')
+        pylab.savefig("tree.png")
         pylab.show()
 
 
@@ -614,12 +653,11 @@ def conll_demo():
 
 
 def conll_file_demo():
-    print('Mass conll_read demo...')
-    graphs = [DependencyGraph(entry)
-              for entry in conll_data2.split('\n\n') if entry]
+    print("Mass conll_read demo...")
+    graphs = [DependencyGraph(entry) for entry in conll_data2.split("\n\n") if entry]
     for graph in graphs:
         tree = graph.tree()
-        print('\n')
+        print("\n")
         tree.pprint()
 
 
@@ -627,13 +665,14 @@ def cycle_finding_demo():
     dg = DependencyGraph(treebank_data)
     print(dg.contains_cycle())
     cyclic_dg = DependencyGraph()
-    cyclic_dg.add_node({'word': None, 'deps': [1], 'rel': 'TOP', 'address': 0})
-    cyclic_dg.add_node({'word': None, 'deps': [2], 'rel': 'NTOP', 'address': 1})
-    cyclic_dg.add_node({'word': None, 'deps': [4], 'rel': 'NTOP', 'address': 2})
-    cyclic_dg.add_node({'word': None, 'deps': [1], 'rel': 'NTOP', 'address': 3})
-    cyclic_dg.add_node({'word': None, 'deps': [3], 'rel': 'NTOP', 'address': 4})
+    cyclic_dg.add_node({"word": None, "deps": [1], "rel": "TOP", "address": 0})
+    cyclic_dg.add_node({"word": None, "deps": [2], "rel": "NTOP", "address": 1})
+    cyclic_dg.add_node({"word": None, "deps": [4], "rel": "NTOP", "address": 2})
+    cyclic_dg.add_node({"word": None, "deps": [1], "rel": "NTOP", "address": 3})
+    cyclic_dg.add_node({"word": None, "deps": [3], "rel": "NTOP", "address": 4})
     print(cyclic_dg.contains_cycle())
 
+
 treebank_data = """Pierre  NNP     2       NMOD
 Vinken  NNP     8       SUB
 ,       ,       2       P
@@ -756,5 +795,5 @@ conll_data2 = """1   Cathy             Cathy             N     N     eigen|ev|ne
 16  .                 .                 Punc  Punc  punt                             15  punct   _  _
 """
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     demo()
