@@ -1,17 +1,14 @@
 # Natural Language Toolkit: Dependency Grammars
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2024 NLTK Project
 # Author: Jason Narad <jason.narad@gmail.com>
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 #
-from __future__ import print_function
 
-import math
 import logging
-
-from six.moves import range
+import math
 
 from nltk.parse.dependencygraph import DependencyGraph
 
@@ -22,7 +19,7 @@ logger = logging.getLogger(__name__)
 #################################################################
 
 
-class DependencyScorerI(object):
+class DependencyScorerI:
     """
     A scorer for calculated the weights on the edges of a weighted
     dependency graph.  This is used by a
@@ -36,15 +33,15 @@ class DependencyScorerI(object):
 
     def __init__(self):
         if self.__class__ == DependencyScorerI:
-            raise TypeError('DependencyScorerI is an abstract interface')
+            raise TypeError("DependencyScorerI is an abstract interface")
 
     def train(self, graphs):
         """
         :type graphs: list(DependencyGraph)
         :param graphs: A list of dependency graphs to train the scorer.
-        Typically the edges present in the graphs can be used as
-        positive training examples, and the edges not present as negative
-        examples.
+            Typically the edges present in the graphs can be used as
+            positive training examples, and the edges not present as negative
+            examples.
         """
         raise NotImplementedError()
 
@@ -52,27 +49,30 @@ class DependencyScorerI(object):
         """
         :type graph: DependencyGraph
         :param graph: A dependency graph whose set of edges need to be
-        scored.
+            scored.
         :rtype: A three-dimensional list of numbers.
         :return: The score is returned in a multidimensional(3) list, such
-        that the outer-dimension refers to the head, and the
-        inner-dimension refers to the dependencies.  For instance,
-        scores[0][1] would reference the list of scores corresponding to
-        arcs from node 0 to node 1.  The node's 'address' field can be used
-        to determine its number identification.
+            that the outer-dimension refers to the head, and the
+            inner-dimension refers to the dependencies.  For instance,
+            scores[0][1] would reference the list of scores corresponding to
+            arcs from node 0 to node 1.  The node's 'address' field can be used
+            to determine its number identification.
 
         For further illustration, a score list corresponding to Fig.2 of
-        Keith Hall's 'K-best Spanning Tree Parsing' paper:
+        Keith Hall's 'K-best Spanning Tree Parsing' paper::
+
               scores = [[[], [5],  [1],  [1]],
                        [[], [],   [11], [4]],
                        [[], [10], [],   [5]],
                        [[], [8],  [8],  []]]
+
         When used in conjunction with a MaxEntClassifier, each score would
         correspond to the confidence of a particular edge being classified
         with the positive training examples.
         """
         raise NotImplementedError()
 
+
 #################################################################
 # NaiveBayesDependencyScorer
 #################################################################
@@ -117,17 +117,17 @@ class NaiveBayesDependencyScorer(DependencyScorerI):
         for graph in graphs:
             for head_node in graph.nodes.values():
                 for child_index, child_node in graph.nodes.items():
-                    if child_index in head_node['deps']:
+                    if child_index in head_node["deps"]:
                         label = "T"
                     else:
                         label = "F"
                     labeled_examples.append(
                         (
                             dict(
-                                a=head_node['word'],
-                                b=head_node['tag'],
-                                c=child_node['word'],
-                                d=child_node['tag'],
+                                a=head_node["word"],
+                                b=head_node["tag"],
+                                c=child_node["word"],
+                                d=child_node["tag"],
                             ),
                             label,
                         )
@@ -152,13 +152,11 @@ class NaiveBayesDependencyScorer(DependencyScorerI):
         for head_node in graph.nodes.values():
             for child_node in graph.nodes.values():
                 edges.append(
-                    (
-                        dict(
-                            a=head_node['word'],
-                            b=head_node['tag'],
-                            c=child_node['word'],
-                            d=child_node['tag'],
-                        )
+                    dict(
+                        a=head_node["word"],
+                        b=head_node["tag"],
+                        c=child_node["word"],
+                        d=child_node["tag"],
                     )
                 )
 
@@ -167,9 +165,9 @@ class NaiveBayesDependencyScorer(DependencyScorerI):
         row = []
         count = 0
         for pdist in self.classifier.prob_classify_many(edges):
-            logger.debug('%.4f %.4f', pdist.prob('T'), pdist.prob('F'))
+            logger.debug("%.4f %.4f", pdist.prob("T"), pdist.prob("F"))
             # smoothing in case the probability = 0
-            row.append([math.log(pdist.prob("T")+0.00000000001)])
+            row.append([math.log(pdist.prob("T") + 0.00000000001)])
             count += 1
             if count == len(graph.nodes):
                 edge_scores.append(row)
@@ -184,21 +182,24 @@ class NaiveBayesDependencyScorer(DependencyScorerI):
 # A short class necessary to show parsing example from paper
 class DemoScorer(DependencyScorerI):
     def train(self, graphs):
-        print('Training...')
+        print("Training...")
 
     def score(self, graph):
         # scores for Keith Hall 'K-best Spanning Tree Parsing' paper
-        return [[[], [5],  [1],  [1]],
-                [[], [],   [11], [4]],
-                [[], [10], [],   [5]],
-                [[], [8],  [8],  []]]
+        return [
+            [[], [5], [1], [1]],
+            [[], [], [11], [4]],
+            [[], [10], [], [5]],
+            [[], [8], [8], []],
+        ]
+
 
 #################################################################
 # Non-Projective Probabilistic Parsing
 #################################################################
 
 
-class ProbabilisticNonprojectiveParser(object):
+class ProbabilisticNonprojectiveParser:
     """A probabilistic non-projective dependency parser.
 
     Nonprojective dependencies allows for "crossing branches" in the parse tree
@@ -229,7 +230,6 @@ class ProbabilisticNonprojectiveParser(object):
     1
 
     Rule based example
-    ------------------
 
     >>> from nltk.grammar import DependencyGrammar
 
@@ -248,11 +248,12 @@ class ProbabilisticNonprojectiveParser(object):
     4
 
     """
+
     def __init__(self):
         """
         Creates a new non-projective parser.
         """
-        logging.debug('initializing prob. nonprojective...')
+        logging.debug("initializing prob. nonprojective...")
 
     def train(self, graphs, dependency_scorer):
         """
@@ -294,12 +295,12 @@ class ProbabilisticNonprojectiveParser(object):
         :type g_graph, b_graph, c_graph: DependencyGraph
         :param g_graph, b_graph, c_graph: Graphs which need to be updated.
         """
-        logger.debug('Collapsing nodes...')
+        logger.debug("Collapsing nodes...")
         # Collapse all cycle nodes into v_n+1 in G_Graph
         for cycle_node_index in cycle_path:
             g_graph.remove_by_address(cycle_node_index)
         g_graph.add_node(new_node)
-        g_graph.redirect_arcs(cycle_path, new_node['address'])
+        g_graph.redirect_arcs(cycle_path, new_node["address"])
 
     def update_edge_scores(self, new_node, cycle_path):
         """
@@ -311,24 +312,20 @@ class ProbabilisticNonprojectiveParser(object):
         :type cycle_path: A list of integers.
         :param cycle_path: A list of node addresses that belong to the cycle.
         """
-        logger.debug('cycle %s', cycle_path)
+        logger.debug("cycle %s", cycle_path)
 
         cycle_path = self.compute_original_indexes(cycle_path)
 
-        logger.debug('old cycle %s', cycle_path)
-        logger.debug('Prior to update: %s', self.scores)
+        logger.debug("old cycle %s", cycle_path)
+        logger.debug("Prior to update: %s", self.scores)
 
         for i, row in enumerate(self.scores):
             for j, column in enumerate(self.scores[i]):
                 logger.debug(self.scores[i][j])
-                if (
-                    j in cycle_path
-                    and i not in cycle_path
-                    and self.scores[i][j]
-                ):
+                if j in cycle_path and i not in cycle_path and self.scores[i][j]:
                     subtract_val = self.compute_max_subtract_score(j, cycle_path)
 
-                    logger.debug('%s - %s', self.scores[i][j], subtract_val)
+                    logger.debug("%s - %s", self.scores[i][j], subtract_val)
 
                     new_vals = []
                     for cur_val in self.scores[i][j]:
@@ -341,7 +338,7 @@ class ProbabilisticNonprojectiveParser(object):
                 if i in cycle_path and j in cycle_path:
                     self.scores[i][j] = []
 
-        logger.debug('After update: %s', self.scores)
+        logger.debug("After update: %s", self.scores)
 
     def compute_original_indexes(self, new_indexes):
         """
@@ -353,7 +350,7 @@ class ProbabilisticNonprojectiveParser(object):
 
         :type new_indexes: A list of integers.
         :param new_indexes: A list of node addresses to check for
-        subsumed nodes.
+            subsumed nodes.
         """
         swapped = True
         while swapped:
@@ -378,10 +375,10 @@ class ProbabilisticNonprojectiveParser(object):
 
         :type column_index: integer.
         :param column_index: A index representing the column of incoming arcs
-        to a particular node being updated
+            to a particular node being updated
         :type cycle_indexes: A list of integers.
         :param cycle_indexes: Only arcs from cycle nodes are considered.  This
-        is a list of such nodes addresses.
+            is a list of such nodes addresses.
         """
         max_score = -100000
         for row_index in cycle_indexes:
@@ -397,20 +394,21 @@ class ProbabilisticNonprojectiveParser(object):
 
         :type node_index: integer.
         :param node_index: The address of the 'destination' node,
-        the node that is arced to.
+            the node that is arced to.
         """
         originals = self.compute_original_indexes([node_index])
-        logger.debug('originals: %s', originals)
+        logger.debug("originals: %s", originals)
 
         max_arc = None
         max_score = None
         for row_index in range(len(self.scores)):
             for col_index in range(len(self.scores[row_index])):
-                # print self.scores[row_index][col_index]
-                if col_index in originals and (max_score is None or self.scores[row_index][col_index] > max_score):
+                if col_index in originals and (
+                    max_score is None or self.scores[row_index][col_index] > max_score
+                ):
                     max_score = self.scores[row_index][col_index]
                     max_arc = row_index
-                    logger.debug('%s, %s', row_index, col_index)
+                    logger.debug("%s, %s", row_index, col_index)
 
         logger.debug(max_score)
 
@@ -428,7 +426,9 @@ class ProbabilisticNonprojectiveParser(object):
         max_orig = None
         for row_index in range(len(self.scores)):
             for col_index in range(len(self.scores[row_index])):
-                if col_index in originals and (max_score is None or self.scores[row_index][col_index] > max_score):
+                if col_index in originals and (
+                    max_score is None or self.scores[row_index][col_index] > max_score
+                ):
                     max_score = self.scores[row_index][col_index]
                     max_arc = row_index
                     max_orig = col_index
@@ -455,27 +455,15 @@ class ProbabilisticNonprojectiveParser(object):
         g_graph = DependencyGraph()
         for index, token in enumerate(tokens):
             g_graph.nodes[index + 1].update(
-                {
-                    'word': token,
-                    'tag': tags[index],
-                    'rel': 'NTOP',
-                    'address': index + 1,
-                }
+                {"word": token, "tag": tags[index], "rel": "NTOP", "address": index + 1}
             )
-        #print (g_graph.nodes)
-
 
         # Fully connect non-root nodes in g_graph
         g_graph.connect_graph()
         original_graph = DependencyGraph()
         for index, token in enumerate(tokens):
             original_graph.nodes[index + 1].update(
-                {
-                    'word': token,
-                    'tag': tags[index],
-                    'rel': 'NTOP',
-                    'address': index+1,
-                }
+                {"word": token, "tag": tags[index], "rel": "NTOP", "address": index + 1}
             )
 
         b_graph = DependencyGraph()
@@ -483,43 +471,32 @@ class ProbabilisticNonprojectiveParser(object):
 
         for index, token in enumerate(tokens):
             c_graph.nodes[index + 1].update(
-                {
-                    'word': token,
-                    'tag': tags[index],
-                    'rel': 'NTOP',
-                    'address': index + 1,
-                }
+                {"word": token, "tag": tags[index], "rel": "NTOP", "address": index + 1}
             )
 
         # Assign initial scores to g_graph edges
         self.initialize_edge_scores(g_graph)
         logger.debug(self.scores)
         # Initialize a list of unvisited vertices (by node address)
-        unvisited_vertices = [
-            vertex['address'] for vertex in c_graph.nodes.values()
-        ]
+        unvisited_vertices = [vertex["address"] for vertex in c_graph.nodes.values()]
         # Iterate over unvisited vertices
         nr_vertices = len(tokens)
         betas = {}
         while unvisited_vertices:
             # Mark current node as visited
             current_vertex = unvisited_vertices.pop(0)
-            logger.debug('current_vertex: %s', current_vertex)
+            logger.debug("current_vertex: %s", current_vertex)
             # Get corresponding node n_i to vertex v_i
             current_node = g_graph.get_by_address(current_vertex)
-            logger.debug('current_node: %s', current_node)
+            logger.debug("current_node: %s", current_node)
             # Get best in-edge node b for current node
             best_in_edge = self.best_incoming_arc(current_vertex)
             betas[current_vertex] = self.original_best_arc(current_vertex)
-            logger.debug('best in arc: %s --> %s', best_in_edge, current_vertex)
+            logger.debug("best in arc: %s --> %s", best_in_edge, current_vertex)
             # b_graph = Union(b_graph, b)
             for new_vertex in [current_vertex, best_in_edge]:
                 b_graph.nodes[new_vertex].update(
-                    {
-                        'word': 'TEMP',
-                        'rel': 'NTOP',
-                        'address': new_vertex,
-                    }
+                    {"word": "TEMP", "rel": "NTOP", "address": new_vertex}
                 )
             b_graph.add_arc(best_in_edge, current_vertex)
             # Beta(current node) = b  - stored for parse recovery
@@ -527,21 +504,17 @@ class ProbabilisticNonprojectiveParser(object):
             cycle_path = b_graph.contains_cycle()
             if cycle_path:
                 # Create a new node v_n+1 with address = len(nodes) + 1
-                new_node = {
-                    'word': 'NONE',
-                    'rel': 'NTOP',
-                    'address': nr_vertices + 1,
-                }
+                new_node = {"word": "NONE", "rel": "NTOP", "address": nr_vertices + 1}
                 # c_graph = Union(c_graph, v_n+1)
                 c_graph.add_node(new_node)
                 # Collapse all nodes in cycle C into v_n+1
                 self.update_edge_scores(new_node, cycle_path)
                 self.collapse_nodes(new_node, cycle_path, g_graph, b_graph, c_graph)
                 for cycle_index in cycle_path:
-                    c_graph.add_arc(new_node['address'], cycle_index)
+                    c_graph.add_arc(new_node["address"], cycle_index)
                     # self.replaced_by[cycle_index] = new_node['address']
 
-                self.inner_nodes[new_node['address']] = cycle_path
+                self.inner_nodes[new_node["address"]] = cycle_path
 
                 # Add v_n+1 to list of unvisited vertices
                 unvisited_vertices.insert(0, nr_vertices + 1)
@@ -553,38 +526,39 @@ class ProbabilisticNonprojectiveParser(object):
                 for cycle_node_address in cycle_path:
                     b_graph.remove_by_address(cycle_node_address)
 
-            logger.debug('g_graph: %s', g_graph)
-            logger.debug('b_graph: %s', b_graph)
-            logger.debug('c_graph: %s', c_graph)
-            logger.debug('Betas: %s', betas)
-            logger.debug('replaced nodes %s', self.inner_nodes)
+            logger.debug("g_graph: %s", g_graph)
+            logger.debug("b_graph: %s", b_graph)
+            logger.debug("c_graph: %s", c_graph)
+            logger.debug("Betas: %s", betas)
+            logger.debug("replaced nodes %s", self.inner_nodes)
 
         # Recover parse tree
-        logger.debug('Final scores: %s', self.scores)
+        logger.debug("Final scores: %s", self.scores)
 
-        logger.debug('Recovering parse...')
+        logger.debug("Recovering parse...")
         for i in range(len(tokens) + 1, nr_vertices + 1):
             betas[betas[i][1]] = betas[i]
 
-        logger.debug('Betas: %s', betas)
+        logger.debug("Betas: %s", betas)
         for node in original_graph.nodes.values():
             # TODO: It's dangerous to assume that deps it a dictionary
             # because it's a default dictionary. Ideally, here we should not
             # be concerned how dependencies are stored inside of a dependency
             # graph.
-            node['deps'] = {}
+            node["deps"] = {}
         for i in range(1, len(tokens) + 1):
             original_graph.add_arc(betas[i][0], betas[i][1])
 
-        logger.debug('Done.')
+        logger.debug("Done.")
         yield original_graph
 
+
 #################################################################
 # Rule-based Non-Projective Parser
 #################################################################
 
 
-class NonprojectiveDependencyParser(object):
+class NonprojectiveDependencyParser:
     """
     A non-projective, rule-based, dependency parser.  This parser
     will return the set of all possible non-projective parses based on
@@ -623,21 +597,21 @@ class NonprojectiveDependencyParser(object):
 
         for index, token in enumerate(tokens):
             self._graph.nodes[index] = {
-                'word': token,
-                'deps': [],
-                'rel': 'NTOP',
-                'address': index,
+                "word": token,
+                "deps": [],
+                "rel": "NTOP",
+                "address": index,
             }
 
         for head_node in self._graph.nodes.values():
             deps = []
-            for dep_node in self._graph.nodes.values()  :
+            for dep_node in self._graph.nodes.values():
                 if (
-                    self._grammar.contains(head_node['word'], dep_node['word'])
-                    and head_node['word'] != dep_node['word']
+                    self._grammar.contains(head_node["word"], dep_node["word"])
+                    and head_node["word"] != dep_node["word"]
                 ):
-                    deps.append(dep_node['address'])
-            head_node['deps'] = deps
+                    deps.append(dep_node["address"])
+            head_node["deps"] = deps
 
         # Create lattice of possible heads
         roots = []
@@ -659,7 +633,7 @@ class NonprojectiveDependencyParser(object):
 
             # Traverse lattice
             analyses = []
-            for root in roots:
+            for _ in roots:
                 stack = []
                 analysis = [[] for i in range(len(possible_heads))]
             i = 0
@@ -711,22 +685,19 @@ class NonprojectiveDependencyParser(object):
             graph = DependencyGraph()
             graph.root = graph.nodes[analysis.index(-1) + 1]
 
-            for address, (token, head_index) in enumerate(zip(tokens, analysis), start=1):
+            for address, (token, head_index) in enumerate(
+                zip(tokens, analysis), start=1
+            ):
                 head_address = head_index + 1
 
                 node = graph.nodes[address]
-                node.update(
-                    {
-                        'word': token,
-                        'address': address,
-                    }
-                )
+                node.update({"word": token, "address": address})
 
                 if head_address == 0:
-                    rel = 'ROOT'
+                    rel = "ROOT"
                 else:
-                    rel = ''
-                graph.nodes[head_index + 1]['deps'][rel].append(address)
+                    rel = ""
+                graph.nodes[head_index + 1]["deps"][rel].append(address)
 
             # TODO: check for cycles
             yield graph
@@ -736,6 +707,7 @@ class NonprojectiveDependencyParser(object):
 # Demos
 #################################################################
 
+
 def demo():
     # hall_demo()
     nonprojective_conll_parse_demo()
@@ -745,39 +717,56 @@ def demo():
 def hall_demo():
     npp = ProbabilisticNonprojectiveParser()
     npp.train([], DemoScorer())
-    for parse_graph in npp.parse(['v1', 'v2', 'v3'], [None, None, None]):
+    for parse_graph in npp.parse(["v1", "v2", "v3"], [None, None, None]):
         print(parse_graph)
 
 
 def nonprojective_conll_parse_demo():
     from nltk.parse.dependencygraph import conll_data2
 
-    graphs = [
-        DependencyGraph(entry) for entry in conll_data2.split('\n\n') if entry
-    ]
+    graphs = [DependencyGraph(entry) for entry in conll_data2.split("\n\n") if entry]
     npp = ProbabilisticNonprojectiveParser()
     npp.train(graphs, NaiveBayesDependencyScorer())
-    for parse_graph in npp.parse(['Cathy', 'zag', 'hen', 'zwaaien', '.'], ['N', 'V', 'Pron', 'Adj', 'N', 'Punc']):
+    for parse_graph in npp.parse(
+        ["Cathy", "zag", "hen", "zwaaien", "."], ["N", "V", "Pron", "Adj", "N", "Punc"]
+    ):
         print(parse_graph)
 
 
 def rule_based_demo():
     from nltk.grammar import DependencyGrammar
 
-    grammar = DependencyGrammar.fromstring("""
+    grammar = DependencyGrammar.fromstring(
+        """
     'taught' -> 'play' | 'man'
     'man' -> 'the' | 'in'
     'in' -> 'corner'
     'corner' -> 'the'
     'play' -> 'golf' | 'dachshund' | 'to'
     'dachshund' -> 'his'
-    """)
+    """
+    )
     print(grammar)
     ndp = NonprojectiveDependencyParser(grammar)
-    graphs = ndp.parse(['the', 'man', 'in', 'the', 'corner', 'taught', 'his', 'dachshund', 'to', 'play', 'golf'])
-    print('Graphs:')
+    graphs = ndp.parse(
+        [
+            "the",
+            "man",
+            "in",
+            "the",
+            "corner",
+            "taught",
+            "his",
+            "dachshund",
+            "to",
+            "play",
+            "golf",
+        ]
+    )
+    print("Graphs:")
     for graph in graphs:
         print(graph)
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     demo()
